import logging
import sys
from typing import Mapping

class LLLogger:
    def __init__(self, loggerName:str):
        self.logger = logging.getLogger(loggerName)

        self.logger.setLevel('INFO')
        fmt = '{"time": "%(asctime)s",  "hostname": "%(filename)s",  "level": "%(levelname)s", "msg": "%(message)s",  "name": "%(name)s", "pid": "%(process)d"}'
        fmt_date = '%Y-%m-%dT%T%Z'
        formatter = logging.Formatter(fmt, fmt_date)
        handler = logging.StreamHandler(stream=sys.stdout)
        handler.setFormatter(formatter)

        self.logger.addHandler(handler)

    def info(self, msg: object, *args, **kwargs):
        self.logger.info(msg, *args, **kwargs)

    def warn(self, msg: object, *args, **kwargs):
        self.logger.warn(msg, *args, **kwargs)

    def error(self, msg: object, *args, **kwargs):
        self.logger.error(msg, *args, **kwargs)




import os

LX_BASE_URL = os.environ.get("LX_BASE_URL", "https://api.lytix.co")
LX_API_KEY = os.environ.get("LX_API_KEY")

def validateEnvVars(): 
    if (LX_BASE_URL is None):
       print("LX_BASE_URL is not set")
    if (LX_API_KEY is None):
        print("LX_API_KEY is not set")


"""
Always validate when importing this file
@note This will not hard break, just print an error
"""
validateEnvVars()


import requests
from urllib.parse import urljoin
from lytix_py.envVars import LX_API_KEY, LX_BASE_URL
import os.path

"""
 Main class to collect and report metrics
 back to HQ
"""
class _MetricCollector:
    _apiKey: str = None
    _baseURL: str = None
    # _logger: str = None

    def __init__(self):
        self._apiKey = LX_API_KEY
        self._baseURL = urljoin(LX_BASE_URL, "/v1/metrics")
        pass


    """
    Interal wrapper to send a post request
    """
    def _sendPostRequest(self, endpoint: str, body: dict):
        url = os.path.join(self._baseURL, endpoint)
        headers = {
            "Content-Type": "application/json",
            "lx-api-key": self._apiKey,
        }
        response = requests.post(url, headers=headers, json=body)
        if (response.status_code != 200):
            print(f"Failed to send post request to {url} with status code {response.status_code} and response {response.text}")

    """
    Increment a given metric
    """
    def increment(self, metricName: str, metricValue: int = 1, metricMetadata: dict = None):
        if metricMetadata is None:
            metricMetadata = {}
        body = {
            "metricName": metricName,
            "metricValue": metricValue,
            "metadata": metricMetadata
        }
        self._sendPostRequest("increment", body)

    """
    Capture a model input/output
    """
    def captureModelIO(self, modelName: str, userInput: str, modelOutput: str, metricMetadata: dict = None):
        if metricMetadata is None:
            metricMetadata = {}
        body = {
            "modelName": modelName,
            "userInput": userInput,
            "modelOutput": modelOutput,
            "metadata": metricMetadata
        }
        self._sendPostRequest("modelIO", body)

MetricCollector = _MetricCollector()

